@@ -24,6 +24,10 @@
 #endif
 
 #include <gnome.h>
+
+
+#include <gconf/gconf-client.h>
+
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -33,6 +37,7 @@
 #include <esd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <stdarg.h>
 
 #include "grec.h"
 #include "gui.h"
@@ -60,19 +65,95 @@ gint repeat_counter = 0;
 gchar* active_file = NULL;
 gboolean default_file = FALSE;
 gboolean file_changed = FALSE;
+gboolean show_message = FALSE;
 
 static guint play_id;
 static guint record_id;
 
+extern gboolean able_to_record;
+
+static void
+on_dontshowagain_dialog_destroy_activate (GtkWidget* widget,
+					  gpointer checkbutton)
+{
+	GConfClient *client;
+	gboolean stat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
+	
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, "/apps/gnome-sound-recorder/show-warning-messages", !stat, NULL);
+	g_object_unref (client);
+}
+
+gboolean
+check_for_sox (void)
+{
+	char *p;
+	gboolean show_warningmess;
+	GConfClient *client;
+	
+	if (sox_command == NULL) {
+		return FALSE;
+	}
+
+	client = gconf_client_get_default ();
+	show_warningmess = gconf_client_get_bool (client,
+						  "/apps/gnome-sound-recorder/show-warning-messages", NULL);
+	g_object_unref (client);
+
+	p = g_find_program_in_path (sox_command);
+	if (p == NULL) {
+		able_to_record = FALSE;
+		gtk_widget_set_sensitive (GTK_WIDGET (grecord_widgets.Record_button), FALSE);
+		if (show_warningmess) {
+			GtkWidget* dont_show_again_checkbutton = gtk_check_button_new_with_label (_("Don't show this message again."));
+			
+			gchar* show_mess = g_strdup_printf (_("Could not find '%s'.\nSet the correct path to sox in"
+							      "preferences under the tab 'paths'.\n\nIf you don't have"
+							      " sox, you will not be able to record or do any effects."),
+							    sox_command);
+			GtkWidget* mess = gtk_message_dialog_new (NULL,
+								  GTK_DIALOG_MODAL,	
+								  GTK_MESSAGE_WARNING,
+								  GTK_BUTTONS_OK,
+								  show_mess);
+			g_free (show_mess);
+			
+			gtk_widget_show (dont_show_again_checkbutton);
+			gtk_container_add (GTK_CONTAINER (GTK_DIALOG (mess)->vbox), dont_show_again_checkbutton);
+			
+			/* Connect a signal on ok-button, so we can get the stat on the checkbutton */
+			g_signal_connect (mess, "destroy",
+					  G_CALLBACK (on_dontshowagain_dialog_destroy_activate), dont_show_again_checkbutton);
+			
+			gtk_dialog_run (GTK_DIALOG (mess));
+			gtk_widget_destroy (mess);
+		}
+
+		able_to_record = FALSE;
+		gtk_widget_set_sensitive (grecord_widgets.Record_button, FALSE);
+		return FALSE;
+	} else {
+		g_free (p);
+	}
+
+	return TRUE;
+}
+
 /* ------------------- Callbacks ------------------------------- */
 void
 on_record_activate_cb (GtkWidget* widget, gpointer data)
 {
+	show_message = TRUE;
 	/* Check if the sounddevice is ready */
 	if (!check_if_sounddevice_ready ())
 		return;
 
+	if (check_for_sox () == FALSE) {
+		return;
+	}
+	
 	grecord_set_sensitive_progress ();
+	save_set_sensitive (TRUE);
 	file_changed = TRUE;
 
 	/* Reset record-time and stuff */
@@ -111,9 +192,26 @@ on_play_activate_cb (GtkWidget* widget, gpointer data)
 
 	PlayEng.pid = fork ();
 	if (PlayEng.pid == 0) {
+#if 0
+		int i;
+		
 		/* Play file */
+		if (playrepeat == TRUE &&
+		    playrepeatforever == FALSE) {
+			for (i = 0; i < playxtimes; i++) {
+				g_print ("%d of %d: Playing %s\n", i, playxtimes, active_file);
+				play_sound (active_file);
+			}
+		} else if (playrepeat == TRUE &&
+			   playrepeatforever == TRUE) {
+			while (1) {
+				play_sound (active_file);
+			}
+		} else if (playrepeat == FALSE) {
+			play_sound (active_file);
+		}
+#endif
 		play_sound (active_file);
-
 		_exit (0);
 	}
 	else if (PlayEng.pid == -1)
@@ -132,8 +230,9 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 {
 	gchar* temp_string1 = NULL;
 	gchar* temp_string2 = NULL;
+	mode_t old_mask;
 
-	gnome_appbar_pop (GNOME_APPBAR (grecord_widgets.appbar));
+	gnome_appbar_clear_stack (GNOME_APPBAR (grecord_widgets.appbar));
 
 	if (RecEng.is_running) {
 		temp_string1 = g_strconcat ("-r ", samplerate, NULL);
@@ -158,26 +257,26 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 	}
 
 	if (RecEng.is_running) {
-		gchar* command;
 		gchar* temp_string;
-		gchar* tfile1 = g_concat_dir_and_file (temp_dir, temp_filename_record);
-		gchar* tfile2 = g_concat_dir_and_file (temp_dir, temp_filename_play);
-		
-		command = g_strconcat (sox_command, " ", temp_string1, " ", temp_string2, " ",
-				       "-w ", "-s ", tfile1, " ",
-				       tfile2, NULL);
+		gchar* tfile1 = g_build_filename (temp_dir, 
+						  temp_filename_record, NULL);
+		gchar* tfile2 = g_build_filename (temp_dir, 
+						  temp_filename_play, NULL);
 		
 		kill (RecEng.pid, SIGKILL);
 		waitpid (RecEng.pid, NULL, WUNTRACED);
 
 		RecEng.is_running = FALSE;
 
-		run_command (command, _("Converting file..."));
+		old_mask = umask(0077);
+		run_command (_("Converting file..."), sox_command, temp_string1,
+			     temp_string2, "-w", "-s", tfile1, tfile2, NULL);
+		umask(old_mask);
 
-		temp_string = g_concat_dir_and_file (temp_dir, temp_filename_play);
+		temp_string = g_build_filename (temp_dir, 
+						temp_filename_play, NULL);
 
 		g_free (temp_string);
-		g_free (command);
 		g_free (tfile1);
 		g_free (tfile2);
 	}
@@ -192,15 +291,15 @@ on_new_activate_cb (GtkWidget* widget, gpointer data)
 	gint choice;
 	gchar* string = NULL;
 	gchar* temp_string = NULL;
-	gchar* file1 = g_concat_dir_and_file (temp_dir, temp_filename_record);
-	gchar* file2 = g_concat_dir_and_file (temp_dir, temp_filename_play);
-	gchar* file3 = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	gchar* file1 = g_build_filename (temp_dir, temp_filename_record, NULL);
+	gchar* file2 = g_build_filename (temp_dir, temp_filename_play, NULL);
+	gchar* file3 = g_build_filename (temp_dir, temp_filename_backup, NULL);
 
 	if (PlayEng.is_running || RecEng.is_running || convert_is_running)
 		on_stop_activate_cb (widget, data);
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Cancel"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -218,12 +317,17 @@ on_new_activate_cb (GtkWidget* widget, gpointer data)
 	g_free (file2);
 	g_free (file3);
 
+	
 	default_file = TRUE;
-
-	active_file = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	
+	if (active_file != NULL) {
+		g_free (active_file);
+	}
+	active_file = g_build_filename (temp_dir, temp_filename_play, NULL);
 	file_changed = FALSE;
-
-	set_min_sec_time (get_play_time (active_file), TRUE);
+	save_set_sensitive (FALSE);
+	
+	set_min_sec_time (get_play_time (active_file));
 
 	grecord_set_sensitive_nofile ();
 
@@ -263,6 +367,7 @@ on_open_activate_cb (GtkWidget* widget, gpointer data)
 {
 	gint choice;
 	static GtkWidget* filesel = NULL;
+
 	if (filesel) {
 		if (filesel->window == NULL)
 			return;
@@ -273,7 +378,7 @@ on_open_activate_cb (GtkWidget* widget, gpointer data)
 	}
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Cancel open"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -284,20 +389,16 @@ on_open_activate_cb (GtkWidget* widget, gpointer data)
 
 	filesel = gtk_file_selection_new (_("Select a sound file"));
 
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			    "clicked", 
-			    GTK_SIGNAL_FUNC (store_filename),
-			    filesel);
+	g_signal_connect ((GTK_FILE_SELECTION (filesel)->ok_button),
+			  "clicked", G_CALLBACK (store_filename),
+			  filesel);
 
-	gtk_signal_connect (GTK_OBJECT (filesel),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &filesel);
+	g_signal_connect (filesel, "destroy", G_CALLBACK (gtk_widget_destroyed),
+			  &filesel);
 
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-				   "clicked", 
-				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				   (gpointer) filesel);
+	g_signal_connect_swapped ((GTK_FILE_SELECTION (filesel)->cancel_button),
+				  "clicked", G_CALLBACK (gtk_widget_destroy),
+				  (gpointer) filesel);
 
 	gtk_widget_show (filesel);
 }
@@ -327,13 +428,12 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 {
 	gint choice;
 	gchar* tfile;
-	gchar* command;
 
 	if (PlayEng.is_running || RecEng.is_running)
 		on_stop_activate_cb (widget, data);
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Cancel"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -343,12 +443,10 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 	}
 
 	/* User didn't want to save; copy the backup file to the changed file (active file) */
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	tfile = g_build_filename (temp_dir, temp_filename_backup, NULL);
 	
-	if (g_file_exists (tfile)) {
-		command = g_strconcat ("cp -f ", tfile, " ", active_file, NULL);
-		system (command);
-		g_free (command);
+	if (g_file_test (tfile, G_FILE_TEST_EXISTS)) {
+		execlp ("cp", "cp", "-f", tfile, active_file, NULL);
 	}
 
 	gtk_main_quit ();
@@ -356,10 +454,10 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 	remove (tfile);
 	g_free (tfile);
 
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_record);
+	tfile = g_build_filename (temp_dir, temp_filename_record, NULL);
 	remove (tfile);
 	g_free (tfile);
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	tfile = g_build_filename (temp_dir, temp_filename_play, NULL);
 	remove (tfile);
 	g_free (tfile);
 }
@@ -379,10 +477,8 @@ on_preferences_activate_cb (GtkWidget* widget, gpointer data)
 	}
 	props = create_grecord_propertybox ();
 	
-	gtk_signal_connect (GTK_OBJECT (props),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &props);
+	g_signal_connect (props, "destroy", G_CALLBACK (gtk_widget_destroyed),
+			  &props);
 	gtk_widget_show (props);
 }
 
@@ -392,30 +488,45 @@ on_about_activate_cb (GtkWidget* widget, gpointer data)
 	static GtkWidget* about = NULL;
 
 	if (about) {
-		if (about->window == NULL)
-			return;
-		
-		gdk_window_show (about->window);
-		gdk_window_raise (about->window);
+		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
 	
 	about = create_about ();
 
-	gtk_signal_connect (GTK_OBJECT (about),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &about);
+	g_signal_connect (G_OBJECT (about),
+			  "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &about);
 
 	gtk_widget_show (about);
 }
 
+/* These need to bring up an error dialog */
 void
 on_runmixer_activate_cb (GtkWidget* widget, gpointer data)
 {
-	gchar* temp_string = g_strconcat (mixer_command, " &", NULL);
-	system (temp_string);
-	g_free (temp_string);
+	char *mixer_path;
+	char *argv[2] = {NULL, NULL};
+	GError *error = NULL;
+	gboolean ret;
+
+	/* Open the mixer */
+	mixer_path = g_find_program_in_path (DEFAULT_MIXER);
+	if (mixer_path == NULL) {
+		g_warning (_("%s is not installed in the path"), DEFAULT_MIXER);
+		return;
+	}
+	
+	argv[0] = mixer_path;
+	ret = g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, &error);
+	if (ret == FALSE) {
+		g_warning (_("There was an error starting %s: %s"),
+			   mixer_path, error->message);
+		g_error_free (error);
+	}
+
+	g_free (mixer_path);
 }
 
 void 
@@ -431,16 +542,18 @@ on_decrease_speed_activate_cb (GtkWidget* widget, gpointer data)
 void
 on_add_echo_activate_cb (GtkWidget* widget, gpointer data)
 {
-	if (!g_file_exists (active_file))
+	if (!g_file_test (active_file, G_FILE_TEST_EXISTS))
 	    return;
 
 	file_changed = TRUE;
+	save_set_sensitive (TRUE);
 	add_echo (active_file, TRUE);
 }
 
 void
 on_show_time_activate_cb (GtkWidget* widget, gpointer data)
 {
+	GConfClient *client;
 	GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM (widget);
 	gboolean active = item->active;
 
@@ -455,15 +568,15 @@ on_show_time_activate_cb (GtkWidget* widget, gpointer data)
 		gtk_widget_hide (grecord_widgets.timemin_label);
 	}
 
-	show_time = active;
-
-	gnome_config_set_bool ("/grecord/GUI Options/showtime", show_time);
-	gnome_config_sync ();
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, "/apps/gnome-sound-recorder/show-time", active, NULL);
+	g_object_unref (G_OBJECT (client));
 }
 
 void
 on_show_soundinfo_activate_cb (GtkWidget* widget, gpointer data)
 {
+	GConfClient *client;
 	GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM (widget);
 	gboolean active = item->active;
 
@@ -478,10 +591,9 @@ on_show_soundinfo_activate_cb (GtkWidget* widget, gpointer data)
 		gtk_widget_hide (grecord_widgets.nr_of_channels_label);
 	}
 
-	show_soundinfo = active;
-
-	gnome_config_set_bool ("/grecord/GUI Options/showsoundinfo", show_soundinfo);
-	gnome_config_sync ();
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, "/apps/gnome-sound-recorder/show-sound-info", active, NULL);
+	g_object_unref (G_OBJECT (client));
 }
 
 void
@@ -497,12 +609,12 @@ on_redo_activate_cb (GtkWidget* widget, gpointer data)
 void
 on_undoall_activate_cb (GtkWidget* widget, gpointer data)
 {
-	gchar* temp_string = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	gchar* temp_string = g_build_filename (temp_dir, 
+					       temp_filename_backup, NULL);
 
-	if (g_file_exists (temp_string)) {
-		gchar* t = g_strconcat ("cp -f ", temp_string, " ", active_file, NULL);
-		run_command (t, _("Undoing all changes..."));
-		g_free (t);
+	if (g_file_test (temp_string, G_FILE_TEST_EXISTS)) {
+		run_command (_("Undoing all changes..."), "cp", "-f",
+			     temp_string, active_file, NULL);
 	}
 
 	g_free (temp_string);
@@ -524,23 +636,45 @@ record_sound (void)
 	gint func = ESD_RECORD;
 	esd_format_t format = 0;
 
-	gchar* host = NULL;
-	gchar* name = NULL;
+	char *host = NULL;
+	char *name = NULL;
 
 	FILE *target = NULL;
 
 	/* Open the file for writing */
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_record);
+	if (temp_dir == NULL) {
+		g_warning ("temp_dir == NULL\nFixing to /tmp");
+		temp_dir = g_strdup ("/tmp");
+	}
+	
+	tfile = g_build_filename (temp_dir, temp_filename_record, NULL);
 	target = fopen (tfile, "w");
+	if(target == NULL) {
+		g_free (tfile);
+		return;
+	}
+
+	/* Set permissions on new file */
+	if(fchmod (fileno (target), 0600) != 0) {
+		g_free (tfile);
+		return;
+	}
 	g_free (tfile);
 
 	/* Set up bits, channels etc after the preferences */
-	if (esd_channels == 1)
+	if (channels == 1) {
 		esd_channels = ESD_MONO;
-	if (audioformat != 0)
+	}
+	
+	if (audioformat != 0) {
 		bits = ESD_BITS8;
+	}
 
-	rate = atoi (samplerate);
+	if (samplerate == NULL) {
+		rate = 22050;
+	} else {
+		rate = atoi (samplerate);
+	}
 
 	format = bits | esd_channels | mode | func;
 	sock = esd_record_stream_fallback (format, rate, host, name);
@@ -568,53 +702,59 @@ void
 store_filename (GtkFileSelection* selector, gpointer file_selector)
 {
 	GtkWidget* mess;
-	gchar* string = NULL;
-	gchar* temp_string = NULL;
+	char* string = NULL;
+	char* temp_string = NULL;
 	AFfilehandle filename;
 	gint in_audioformat, in_channels, in_rate, in_width;
 	struct stat file;
-	
-	gchar* tempfile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
+	const char* tempfile;
+
+	tempfile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
 
 	if (!stat(tempfile, &file)) {
 		if (S_ISDIR (file.st_mode)) {
-			gchar* show_mess = g_strdup_printf (_("'%s' is a directory.\nPlease select a soundfile to be opened."), tempfile);
-			mess = gnome_message_box_new (show_mess,
-						      GNOME_MESSAGE_BOX_ERROR,
-						      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-			g_free (show_mess);
-			gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+			mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						       GTK_MESSAGE_ERROR,
+						       GTK_BUTTONS_OK,
+						       _("'%s' is a folder.\nPlease select a sound file to be opened."),
+						       tempfile);
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
+
 			gtk_widget_show (mess);
 			return;
 		}
-	}
-	else if (errno == ENOENT) {
-		gchar* show_mess = g_strdup_printf (_("File '%s' doesn't exist.\nPlease select a existing soundfile to be opened."), tempfile);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		g_free (show_mess);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+	} else if (errno == ENOENT) {
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("File '%s' doesn't exist.\nPlease select an existing sound file to be opened."),
+					    tempfile);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+
 		gtk_widget_show (mess);
 		return;
 	}
 
 	if (!soundfile_supported (tempfile)) {
-		gchar* show_mess = g_strdup_printf (_("File '%s' isn't a valid soundfile."), tempfile);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		g_free (show_mess);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("File '%s isn't a valid sound file."),
+					       tempfile);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_widget_show (mess);
 		return;
 	}
 
+	if (active_file != NULL) {
+		g_free (active_file);
+	}
 	active_file = g_strdup (tempfile);
 	file_changed = FALSE;
+	save_set_sensitive (FALSE);
 	gtk_widget_destroy (GTK_WIDGET (file_selector));
 	
 	grecord_set_sensitive_file ();
@@ -629,7 +769,7 @@ store_filename (GtkFileSelection* selector, gpointer file_selector)
 	afGetSampleFormat (filename, AF_DEFAULT_TRACK, &in_audioformat, &in_width);
 
 	/* Update mainwindow with the new values and set topic */
-	set_min_sec_time (get_play_time (active_file), TRUE);
+	set_min_sec_time (get_play_time (active_file));
 
 	samplerate = g_strdup_printf ("%d", in_rate);
 	if (in_channels == 2)
@@ -662,7 +802,6 @@ store_filename (GtkFileSelection* selector, gpointer file_selector)
 void
 save_filename (GtkFileSelection* selector, gpointer file_selector)
 {
-	gchar* temp_string;
 	gchar* new_file;
 	struct stat file;
 
@@ -671,79 +810,191 @@ save_filename (GtkFileSelection* selector, gpointer file_selector)
 	/* Check if the file already exists */
 	if (!stat(new_file, &file)) {
 		GtkWidget* mess;
-		gchar* show_mess;
-		gint choice;
 
 		if (S_ISDIR (file.st_mode)) {
-			show_mess = g_strdup_printf (_("'%s' is a directory.\nPlease enter another filename."), new_file);
-			mess = gnome_message_box_new (show_mess,
-						      GNOME_MESSAGE_BOX_ERROR,
-						      GNOME_STOCK_BUTTON_OK,
-						      NULL);
-			g_free (show_mess);
-			gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+			mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						       GTK_MESSAGE_ERROR,
+						       GTK_BUTTONS_OK,
+						       _("'%s' is a folder.\nPlease enter another filename."),
+						       new_file);
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
+			
 			gtk_widget_show (mess);
+			g_free (new_file);
 			return;
 		}
-		show_mess = g_strdup_printf (_("File '%s' already exists.\nDo you want to overwrite it?"), new_file);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_WARNING,
-					      GNOME_STOCK_BUTTON_YES,
-					      GNOME_STOCK_BUTTON_NO,
-					      GNOME_STOCK_BUTTON_CANCEL,
-					      NULL);
-		choice = gnome_dialog_run (GNOME_DIALOG (mess));
-		g_free (show_mess);
-		if (choice == 2 || choice == 1)
+		mess = gtk_message_dialog_new (NULL, 0,
+					       GTK_MESSAGE_WARNING,
+					       GTK_BUTTONS_NONE,
+					       _("<b>File '%s' already exists.</b>\nDo you want to overwrite it?"),
+					       new_file);
+		gtk_dialog_add_buttons (GTK_DIALOG (mess),
+					_("Cancel save"), 0,
+					_("Overwrite"), 1,
+					NULL);
+
+		/* Bad hack usage #2 */
+		gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), TRUE);
+		gtk_label_set_line_wrap (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), FALSE);
+		gtk_label_set_justify (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), GTK_JUSTIFY_LEFT);
+
+		switch (gtk_dialog_run (GTK_DIALOG (mess))) {
+		case 0:
+			gtk_widget_destroy (mess);
+			g_free (new_file);
 			return;
+
+		default:
+			gtk_widget_destroy (mess);
+			break;
+		}
 	}
 
 	/* Check if the soundfile is supported */
 	if (!save_sound_file (new_file)) {
-		GtkWidget* mess = gnome_message_box_new (_("Error saving sound file"),
-							 GNOME_MESSAGE_BOX_ERROR,
-							 GNOME_STOCK_BUTTON_OK,
-							 NULL);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+		GtkWidget *mess;
+
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("Error saving '%s'"),
+					       new_file);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+		
 		gtk_widget_show (mess);
+		
 		g_free (new_file);
 		return;
 	}
 
-	if (active_file[1] == '/') {
-		gchar* tempstring;
-		gchar* string = g_strdup ((char *) strrchr (active_file, '/'));
-		string[1] = ' ';
-		tempstring = g_strconcat (_(maintopic), string, NULL);
-		gtk_window_set_title (GTK_WINDOW (grecord_widgets.grecord_window), tempstring);
-		g_free (tempstring);
-		g_free (string);
+	if (active_file != NULL) {
+		g_free (active_file);
 	}
-
-	active_file = g_strdup (new_file);
-	g_free (new_file);
+	active_file = new_file;
 	
 	gtk_widget_destroy (GTK_WIDGET (file_selector));
 
-	temp_string = g_strconcat ((maintopic), active_file, NULL);
-	gtk_window_set_title (GTK_WINDOW (grecord_widgets.grecord_window), temp_string);
-	g_free (temp_string);
-
+	set_window_title (active_file);
 	file_changed = FALSE;
+	save_set_sensitive (FALSE);
 	file_selector = NULL;
 }
 
-gint
-save_dont_or_cancel (void)
+/* Yes I stole this from GEdit. */
+static GtkWidget *
+grec_button_new_with_stock_image (const char *text,
+				  const char *stock_id)
 {
-	GtkWidget* mess = gnome_message_box_new (_("File not saved. Do you want to save it?"),
-						 GNOME_MESSAGE_BOX_QUESTION,
-						 GNOME_STOCK_BUTTON_YES,
-						 GNOME_STOCK_BUTTON_NO,
-						 GNOME_STOCK_BUTTON_CANCEL,
-						 NULL);
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
 
-	return (gnome_dialog_run (GNOME_DIALOG (mess)));
+	button = gtk_button_new ();
+
+	if (GTK_BIN (button)->child) {
+		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+	}
+
+	if (gtk_stock_lookup (stock_id, &item)) {
+		label = gtk_label_new_with_mnemonic (text);
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+		hbox = gtk_hbox_new (FALSE, 2);
+		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		gtk_container_add (GTK_CONTAINER (button), align);
+		gtk_container_add (GTK_CONTAINER (align), hbox);
+		gtk_widget_show_all (align);
+
+		return button;
+	}
+
+	label = gtk_label_new_with_mnemonic (text);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+	
+	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+	gtk_widget_show (label);
+	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+static GtkWidget *
+grec_dialog_add_button (GtkDialog *dialog,
+			const char *text,
+			const char *stock_id,
+			int response_id)
+{
+	GtkWidget *button;
+
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	button = grec_button_new_with_stock_image (text, stock_id);
+	g_return_val_if_fail (button != NULL, NULL);
+
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (button);
+
+	gtk_dialog_add_action_widget (dialog, button, response_id);
+
+	return button;
+}
+
+gint
+save_dont_or_cancel (const char *quit_text)
+{
+	int result;
+	char *filename;
+	char *title;
+	GtkWidget *mess;
+
+	if (active_file == NULL) {
+		filename = g_strdup ("untitled.wav");
+	} else {
+		filename = g_path_get_basename (active_file);
+	}
+	
+	mess = gtk_message_dialog_new (grecord_widgets.grecord_window, 0,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_NONE,
+				       _("<b>Do you want to save the changes you made to \"%s\"?</b>\n"
+					 "\nYour changes will be lost is you don't save them."),
+				       filename);
+	
+	grec_dialog_add_button (GTK_DIALOG (mess),
+				_("Do_n't save"), GTK_STOCK_NO, DONTSAVE);
+	grec_dialog_add_button (GTK_DIALOG (mess),
+				quit_text, GTK_STOCK_CANCEL, CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (mess),
+			       GTK_STOCK_SAVE, SAVE);
+	gtk_dialog_set_default_response (GTK_DIALOG (mess), 0);
+
+	title = g_strdup_printf (_("Save %s?"), filename);
+	gtk_window_set_title (GTK_WINDOW (mess), title);
+	g_free (title);
+	g_free (filename);
+	
+	/* This is a bad hack, it's marked private, but tough */
+	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), TRUE);
+	gtk_label_set_line_wrap (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), FALSE);
+	gtk_label_set_justify (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), GTK_JUSTIFY_LEFT);
+	
+	result = gtk_dialog_run (GTK_DIALOG (mess));
+	gtk_widget_destroy (mess);
+	return result;
 }
 
 gboolean
@@ -751,17 +1002,18 @@ save_sound_file (const gchar* filename)
 {
 	/* Check if the file is default (if it's been recorded) */
 	if (default_file) {
-		gchar* tfile = g_concat_dir_and_file (temp_dir, temp_filename_play);
-		gchar* command = g_strconcat ("cp -f ", tfile, " ", filename, NULL);
+		gchar* tfile = g_build_filename (temp_dir, 
+						 temp_filename_play, NULL);
 		
 		/* Save the file */
-		run_command (command, _("Saving..."));
+		run_command (_("Saving..."), "cp", "-f", tfile, filename, NULL);
 
 		g_free (tfile);
-		g_free (command);
 
 		/* It's saved now */
 		default_file = FALSE;
+		save_set_sensitive (FALSE);
+		file_changed = FALSE;
 	}
 
 	/* No saving is needed, because the changes go directly to the active file; don't worry, */
@@ -783,22 +1035,20 @@ UpdateStatusbarPlay (gboolean begin)
 
 		length = get_play_time (active_file);
 		temp = 1000 / length;
-	}
-	else {
+	} else {
 		if (waitpid (PlayEng.pid, NULL, WNOHANG | WUNTRACED)) {
 			if (playrepeat && playrepeatforever) {
 				on_play_activate_cb (NULL, NULL);
 				return FALSE;
 			}
 			else if (playrepeat && !playrepeatforever) {
-				if (repeat_counter < playxtimes) {
+				if (repeat_counter < playxtimes - 1) {
 					on_play_activate_cb (NULL, NULL);
 					repeat_counter++;
 					return FALSE;
 				}
 				repeat_counter = 0;
 			}
-			
 			counter = 0;
 			PlayEng.is_running = FALSE;
 			grecord_set_sensitive_file ();
@@ -811,7 +1061,7 @@ UpdateStatusbarPlay (gboolean begin)
 
 	countersec++;
 
-	set_min_sec_time (countersec, FALSE);
+	set_min_sec_time (countersec);
 
 	gtk_range_set_adjustment (GTK_RANGE (grecord_widgets.Statusbar), GTK_ADJUSTMENT (gtk_adjustment_new (counter, 0, 1000, 1, 1, 0)));
 /*  	gtk_range_slider_update (GTK_RANGE (grecord_widgets.Statusbar)); */
@@ -834,10 +1084,15 @@ UpdateStatusbarRecord (gboolean begin)
 	}
 
 	/* Timeout */
-	if (counter >= 1000) {
+	if (counter >= record_timeout * 60) {
 		if (stop_on_timeout) {
 			on_stop_activate_cb (NULL, NULL);
-			return TRUE;
+
+			if (save_when_finished == TRUE) {
+				save_dialog ();
+			}
+
+			return FALSE;
 		}
 
 		counter /= 2;
@@ -845,7 +1100,7 @@ UpdateStatusbarRecord (gboolean begin)
 	
 	if (waitpid (RecEng.pid, NULL, WNOHANG | WUNTRACED)) {
 		if (convert_is_running)
-			return FALSE;
+			return TRUE;
 
 		counter = 0;
 		RecEng.is_running = FALSE;
@@ -860,13 +1115,13 @@ UpdateStatusbarRecord (gboolean begin)
 
 	countersec++;
 	
-	set_min_sec_time (countersec, FALSE);
+	set_min_sec_time (countersec);
 	
 	if (popup_warn_mess) {
 		struct stat fileinfo;
-		static gint maxfilesize = -1;
-		static gboolean show_message = TRUE;
-		gchar* filename = g_concat_dir_and_file (temp_dir, temp_filename_record);
+		gint maxfilesize = -1;
+		gchar* filename = g_build_filename (temp_dir, 
+						    temp_filename_record, NULL);
 		
 		if (maxfilesize == -1)
 			maxfilesize = popup_warn_mess_v;
@@ -876,22 +1131,25 @@ UpdateStatusbarRecord (gboolean begin)
 		g_free (filename);
 		
 		if (fileinfo.st_size >= (maxfilesize * 1000000) && show_message) {
-			gchar* message = g_strdup_printf (N_("The size of the current sample is more than %i Mb!"),
-							  (int) (fileinfo.st_size / 1000000) /* In MB */);
-			GtkWidget* mess = gnome_message_box_new (_(message),
-								 GNOME_MESSAGE_BOX_WARNING,
-								 GNOME_STOCK_BUTTON_OK,
-								 NULL);
+			GtkWidget *mess;
+			
+			mess = gtk_message_dialog_new (grecord_widgets.grecord_window,
+						       0, GTK_MESSAGE_WARNING,
+						       GTK_BUTTONS_OK,
+						       _("The size of the current sample is more than\n%i Mb!"),
+						       (int) (fileinfo.st_size / 1000000));
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
 			gtk_widget_show (mess);
-			g_free (message);
 			show_message = FALSE;
 	        }			
 	}
 	
 	if (stop_record) {
 		struct stat fileinfo;
-		static gint maxfilesize = -1;
-		gchar* filename = g_concat_dir_and_file (temp_dir, temp_filename_record);
+		gint maxfilesize = -1;
+		gchar* filename = g_build_filename (temp_dir, 
+						    temp_filename_record, NULL);
 
 		if (maxfilesize == -1)
 			maxfilesize = stop_record_v;
@@ -902,18 +1160,27 @@ UpdateStatusbarRecord (gboolean begin)
 
 		if (fileinfo.st_size >= (maxfilesize * 1000000 /* In MB */ )) {
 		  	on_stop_activate_cb (NULL, NULL);
+
+		if (save_when_finished)
+			save_dialog ();
+
 		        return FALSE;
 		}
 	}
 
-	/* Get the timeout value and convert it to seconds (if it isn't allready done) */
-	if (timeout == 0)
+	/* Get the timeout value and convert it to seconds
+	   (if it isn't already done) */
+	if (timeout == 0) {
 		timeout = record_timeout * 60;
+	}
 
-	counter += (int) 1000 / timeout;
+	counter++;
 
-	gtk_range_set_adjustment (GTK_RANGE (grecord_widgets.Statusbar), GTK_ADJUSTMENT (gtk_adjustment_new (counter, 0, 1000, 1, 1, 0)));
-/*  	gtk_range_slider_update (GTK_RANGE (grecord_widgets.Statusbar)); */
+	gtk_range_set_adjustment (GTK_RANGE (grecord_widgets.Statusbar),
+				  GTK_ADJUSTMENT (gtk_adjustment_new (counter,
+								      0,
+								      timeout,
+								      1, 1, 0)));
 
 	return TRUE;
 }
@@ -922,31 +1189,31 @@ void
 save_dialog (void)
 {
 	GtkWidget* filesel = NULL;
-	gchar* temp_file = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	gchar* temp_file = g_build_filename (temp_dir, 
+					     temp_filename_play, NULL);
 
 	filesel = gtk_file_selection_new (_("Save sound file"));
 
 	if (!g_strcasecmp (active_file, temp_file)) {
 		gchar* home_dir = g_strdup (getenv ("HOME"));
-		gchar* default_file = g_concat_dir_and_file (home_dir, _(temp_filename_play));
+		gchar* d_file = g_build_filename (home_dir, 
+						  _(temp_filename_play),
+						  NULL);
 
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), default_file);
+		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), d_file);
 
 		g_free (home_dir);
-		g_free (default_file);
+		g_free (d_file);
 	}
 	else
 		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), active_file);
 		
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			    "clicked", 
-			    GTK_SIGNAL_FUNC (save_filename),
-			    filesel);
+	g_signal_connect ((GTK_FILE_SELECTION (filesel)->ok_button),
+			  "clicked", G_CALLBACK (save_filename), filesel);
 
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-				   "clicked", 
-				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				   (gpointer) filesel);
+	g_signal_connect_swapped ((GTK_FILE_SELECTION (filesel)->cancel_button),
+				  "clicked", G_CALLBACK (gtk_widget_destroy),
+				  (gpointer) filesel);
 
 	gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);
 	
@@ -959,11 +1226,13 @@ void
 is_file_default (void)
 {
 	gchar* temp_string;
-	temp_string = g_concat_dir_and_file (temp_dir, temp_filename_play);
-	if (!g_strcasecmp (temp_string, active_file))
+	temp_string = g_build_filename (temp_dir, temp_filename_play, NULL);
+
+	if (!g_strcasecmp (temp_string, active_file)) {
 	        default_file = TRUE;
-	else
+	} else {
 		default_file = FALSE;
+	}
 
 	g_free (temp_string);
 }
@@ -983,7 +1252,7 @@ void grecord_set_sensitive_file (void)
 
 void grecord_set_sensitive_nofile (void)
 {
-	gtk_widget_set_sensitive (GTK_WIDGET (grecord_widgets.Record_button), TRUE);
+	gtk_widget_set_sensitive (grecord_widgets.Record_button, able_to_record);
 	gtk_widget_set_sensitive (GTK_WIDGET (grecord_widgets.Play_button), FALSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (grecord_widgets.Stop_button), FALSE);
 }
@@ -1010,9 +1279,55 @@ grecord_set_sensitive_loading (void)
 }
 
 void
-run_command (const gchar* command, const gchar* appbar_comment)
+run_command (const gchar* appbar_comment, ...)
 {
-	gint load_pid;
+	/* Non-GTK+/GNOME types for use with non-GTK+/GNOME
+	   POSIX and ANSI calls.
+	*/
+	pid_t load_pid;
+	va_list ap;
+	size_t cmdline_size = 0;
+	size_t ix;
+	char **cmdline = NULL;
+	char *ptr = NULL;
+	char first_arg = 1;
+	char *cmd_name = NULL;
+
+	/* Count the arguments; Record the address of the first
+	   argument. */
+	
+	va_start (ap, appbar_comment);
+	do {
+		ptr = va_arg (ap, char *);
+		if (ptr) {
+			if (first_arg) {
+				cmd_name = ptr;
+				first_arg = 0;
+			}
+
+			cmdline_size++;
+		}
+	} while (ptr);
+	va_end (ap);
+
+	if (first_arg) {
+		g_error ("Cannot run empty command line.");
+	}
+
+	/* Build the argument list */
+	cmdline = g_malloc ((cmdline_size + 1) * sizeof (char *));
+	va_start (ap, appbar_comment);
+	for (ix = 0; ix < cmdline_size; ix++) {
+		ptr = va_arg (ap, char *);
+		cmdline[ix] = ptr;
+	}
+	va_end (ap);
+
+	/* The last of the strings in {cmdline} is the NULL pointer
+	   required by execvp.
+	*/
+
+	cmdline[cmdline_size] = NULL;
 
 	/* Make the widgets insensitive */
 	grecord_set_sensitive_loading ();
@@ -1022,15 +1337,27 @@ run_command (const gchar* command, const gchar* appbar_comment)
 
         load_pid = fork ();
 	if (load_pid == 0) {
+		/* BUG: The stderr output of this child process
+		   should be captured. If the exit status is
+		   nonzero, the stderr output should be displayed.
+		   Errors can be caused either by execvp ()
+		   failure, or by the command being executed
+		*/
+
 		/* Run the command */
-		system (command);
+		execvp (cmd_name, cmdline);
 
-		/* Finished, exit child process */
-		_exit (0);
-	}
-	else if (load_pid == -1)
+		/* If this is reached, then an error occurred. */
+
+		perror (cmd_name);
+		_exit (1);
+	} else if (load_pid == -1) {
+		g_free (cmdline);
 		g_error (_("Could not fork child process"));
+	}
 
+	g_free (cmdline);
+	
 	convert_is_running = TRUE;
 
 	/* Add a function for checking when process has died */
@@ -1044,10 +1371,10 @@ check_if_loading_finished (gint pid)
 	if (waitpid (pid, NULL, WNOHANG | WUNTRACED)) {
 		
 		/* Show the playtime of the file */
-		set_min_sec_time (get_play_time (active_file), TRUE);
+		set_min_sec_time (get_play_time (active_file));
 		
 		/* Remove the comment from the appbar, because we're finished */
-		gnome_appbar_pop (GNOME_APPBAR (grecord_widgets.appbar));
+		gnome_appbar_clear_stack (GNOME_APPBAR (grecord_widgets.appbar));
 
 		/* Make widgets sensitive again */
 		grecord_set_sensitive_file ();
@@ -1067,7 +1394,7 @@ soundfile_supported (const gchar* filename)
 	gint soundtype = afGetFileFormat (filetype, NULL);
 
 	/* Check if the file exists */
-	if (!g_file_exists (filename))
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
 		return FALSE;
 
 	/* Check if the file is a valid soundfile */
@@ -1080,27 +1407,24 @@ soundfile_supported (const gchar* filename)
 gboolean
 check_if_sounddevice_ready ()
 {
-	/* Reset errno */
-	errno = 0;
-
+	int fd;
+	
 	/* Check if the sounddevice is ready */
-	esd_audio_open ();
+	fd = esd_open_sound (NULL);
 	 
 	/* Sounddevice not ready, tell the user */
-	if (errno != 0) {
+	if (fd < 0) {
 		GtkWidget* mess;
 
-		esd_audio_close ();
-
-		mess = gnome_message_box_new (_("Sounddevice not ready! Please check that there isn't\nanother program running that's using the sounddevice."),
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		gnome_dialog_run (GNOME_DIALOG (mess));
+		mess = gtk_message_dialog_new (NULL, 0,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("The sound device is not ready. Please check that there "
+						 "isn't\nanother program running that is using the device."));
+		gtk_dialog_run (GTK_DIALOG (mess));
 		return FALSE;
 	}
 
-	esd_audio_close ();
-
+	esd_close (fd);
 	return TRUE;
 }
